#![no_std]
#![no_main]
#![feature(type_alias_impl_trait)]
#![feature(never_type)]

mod mouse_hid;

use embassy_executor::Spawner;
use embassy_futures::join::join;
use embassy_time::{Duration, Timer};

use usbd_hid::descriptor::MouseReport;

use crate::mouse_hid::{MouseConfig, MouseHID};

use {defmt_rtt as _, panic_probe as _};

#[embassy_executor::main]
async fn main(_spawner: Spawner) {
    let p = embassy_rp::init(Default::default());

    let mut mouse_hid = MouseHID::new(
        p.USB,
        MouseConfig {
            vendor_id: 0x046d,
            product_id: 0x101b,
            manufacturer: "Logitech",
            product: "Performance Plus M705",
            serial_number: "B14D65DA",
        },
    )
    .await;

    let usb_fut = MouseHID::run_usb().await;

    const PIXEL: i8 = 2;
    const WAIT_TIME: Duration = Duration::from_secs(10);

    // Do stuff with the class!
    let hid_fut = async {
        loop {
            mouse_move(&mut mouse_hid, WAIT_TIME, PIXEL).await;
            mouse_move(&mut mouse_hid, Duration::from_millis(50), -PIXEL).await;
        }
    };

    // Run everything concurrently.
    // If we had made everything `'static` above instead, we could do this using separate tasks instead.
    join(hid_fut, usb_fut).await;
}

async fn mouse_move(mouse_hid: &mut MouseHID<'static>, duration: Duration, v: i8) {
    Timer::after(duration).await;

    let report = MouseReport {
        buttons: 0,
        x: 0,
        y: v,
        wheel: 0,
        pan: 0,
    };

    mouse_hid.mouse_report(report).await;
}
